
#include "search.h"
#include "BasicStamp2Library.h"
#include "MiscMethods.h"

//global variables
//const float tksPI = 2.0; 
const int rand_div=1500;   //divisor for random number to derive reasonable # of ticks to move
const byte trnAng = 45;    //random angle to turn, 15 deg
const byte trnClk = 5;       //rand number of clicks to turn
const int maxTags = 10;       //number of bytes in tag array, number of tags
const int curTagCompare0 = 0;       //subscript 0 for curTag
const int curTagCompare1 = 1;       //subscript 1 for curTag
const int t0=8;         //subscript 0 for tag that was found
const int t1=9;         //subscript 1 for tag that was found

const int random_seeds[] = {6, 99, 67, 17, 8, 52, 43, 31, 2, 132};
const int maxRandomSeed = 10;

#define INT_TO_CHAR '1'-1

#define FIND_PADDLE 0
#define IGNORE_PADDLE 1
#define PADDLE_FOUND 2

//Tag Index Table
const char tag_table[10][2] = {
                {65, 66},
                {67, 81},
                {85, 78}, 
                {86, 78}, 
                {60, 84}, 
                {64, 65}, 
                {67, 78}, 
                {85, 64}, 
                {63, 83}, 
                {82, 81}};

//makes a left or right 15 degree turn
void ran_turn(){
   
   //decide randomly which way to turn
   if((rand()%2)!=0)
   {
      turnRight(trnAng);
      forwardSlow(trnClk);
   }
      
   else 
   {
      turnLeft(trnAng);
      forwardSlow(trnClk);
   }
   
   //block until turn is done, or we run into something
   while(!isMotorDone())
   {
       if(getIR())
       {
          stopMotor();   
       } 
   }

}

//compares two tags to see if they are the same
int compare_tag(char tag[10], char curTag[2]){
   
   //if we found the tag return 1
   if( (tag[t0] == curTag[curTagCompare0])
      && (tag[t1] == curTag[curTagCompare1]) )
    {
      return 1;
   }
   
   //if we didn't find the correct tag return 0
   return 0;
}

   
//search takes an array of tags to find and searches for them in order
void search(byte tags2find[10]){
   
   //setup a random seed array and declare other local vars

   
   char curTag[2];               //holds the current tag to find
   int tag_found;               //boolean to indicate a tag was found
   int sensor_data;            //boolean to indicate we are done searching
   int i, nextTagIndex, h;               //counters
   char lcdLine1[21];            //lcd line holder
   char lcdLine2[21];
   char numberString[2];
   int totalTags = totalTagsToFind(tags2find);
   //initialize local variables
   i=nextTagIndex=h=0;
   sensor_data = 0; 
   tag_found = 0;

   int randomSeedIndex = 0;
   
    //clear LCD and "announce" the search is on
   clearLcd();
   sendToLcd(1,"RFID Search");
   sendToLcd(2,"The search is on!");

   nextTagIndex = getNextTagIndex(tags2find,nextTagIndex);

    //start looking for tags   
   while(nextTagIndex < maxTags)
   {
      makeYetGetPrintString(tags2find, totalTags, lcdLine1, lcdLine2);

      //update status message on LCD
      sendToLcd(3,lcdLine1);
      sendToLcd(4,lcdLine2);
      
      //get next tag identifiers
      curTag[curTagCompare0] = tag_table[nextTagIndex][curTagCompare0];
      curTag[curTagCompare1] = tag_table[nextTagIndex][curTagCompare1];

        //seed random to get a random number of clicks to move
      srand(random_seeds[randomSeedIndex]);
      if(++randomSeedIndex >= maxRandomSeed)
      {
          randomSeedIndex=0;
      }

      findCorrectTag(curTag);

      //the correct tag has been found.

      setStringToAllNulls(lcdLine1);
    
      //write to LCD that the tag was found
      strcat(lcdLine1,"Found tag number:");
      numberString[0] = nextTagIndex+INT_TO_CHAR;
      numberString[1] = '\0';
      strcat(lcdLine1,numberString);    
      setLcdLine2(lcdLine1);
      
      //increment the array subscript to point to the next tag to find
      nextTagIndex++;
      nextTagIndex = getNextTagIndex(tags2find,nextTagIndex);
         
   }

   //all tags have been found. play the mario theme and
   //block until it is finished.
   playMarioTheme();
   sleep(0);
      
}


#define FIND_PADDLE 0
#define IGNORE_PADDLE 1
#define PADDLE_FOUND 2

int getNextTagIndex(byte tagsToFind[10],int searchFromIndex)
{

   for(;searchFromIndex < maxTags; searchFromIndex++)
   {
      if(tagsToFind[searchFromIndex] == FIND_PADDLE)
      {
         break;
      }
   }

   if(searchFromIndex >= maxTags)
   {
      searchFromIndex = maxTags-1;
   }

   return searchFromIndex;
}


int totalTagsToFind(byte tagsToFind[10])
{
   int totalPaddles = 0;
   int index = 0;

   for(index = 0; index < maxTags; index++)
   {
      if(tagsToFind[index] == FIND_PADDLE)
      {
         totalPaddles++;
      }
   }

   return totalPaddles;
}

void findCorrectTag(char curTag[2])
{
    byte clicksToMove;            //specifies clicks to move
    byte rfidStatus;               //holds error from rfid_read
    byte tag [maxTags];               //holds the 10 bytes returned from read_rfid
    byte irData = 0;
    byte tagFound = 0;

    while(!tagFound)
    {
       //If we are on top of something black from the beginning.
       //we should not move

       irData = getIR();

       if(irData != 0)
       {
           //set clicks to move somewhere between 0 and 21
           clicksToMove = (byte)(rand()/rand_div);

           //start off straight
           setStraight();
           forwardMedium(clicksToMove);
   
           //poll sensor while moving to see if there is anything, stop if there is
           while(!isMotorDone())
           {
              irData = getIR();
              if(irData != 0)
              {
                 stopMotor();
                 break;
              } 
   
           }//end !isMotorDone

       }
       
          
       //get reading if we found something
       if(irData !=0)
       {
          //we have a tag underneath the reader
          if(irData == SENSOR_C)
          {

             rfidStatus = readRfid(tag);
      
               //check for errors
             if(rfidStatus == NO_ERROR)
             {
   
                tagFound = compare_tag(tag, curTag);
         
             }//end if NO_ERROR
             else if(rfidStatus == RFID_READ_TIMEOUT_ERROR)
             {
            
                //try again
                rfidStatus = readRfid(tag);
                
                if(rfidStatus == NO_ERROR) 
                {
                      tagFound = compare_tag(tag, curTag);
                }

             }             
          } 
          else
          {
           //turn right or left depending on sensor data
             
          }//end else   
       }
       //if we found nothing, randomly turn right or left and go for a few clicks
       else  
       {
          ran_turn();
       }

    }
   
}
